package com.syc.slm.slmbi.controller;

import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.NetResponseUtils;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.function.Action0;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.servlet.HandlerMapping;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.Map;
import java.util.function.Supplier;

/**
 * slm-bi
 * 抽取公用得controller父类
 *
 * @author : lin
 * @date : 2021-04-06 09:09
 **/
@Slf4j
public class BaseRestController {

    @Value("${dc_client.host}")
    protected String dcClientHost;
    @Value("${dc_client.interfaces.positionListUrl}")
    protected String positionListUrl;
    @Value("${dc_client.interfaces.deptListUrl}")
    protected String deptListUrl;
    @Value("${dc_client.interfaces.userTree}")
    protected String userTree;
    @Value("${dc_client.interfaces.variableList}")
    protected String variableList;
    @Value("${dc_client.interfaces.deviceList}")
    protected String deviceList;

    @Value("${dc_client.interfaces.deptsRoles}")
    protected String deptsRoles;

    @Value("${dc_client.interfaces.businessFiledData}")
    protected String businessFiledData;





    @ModelAttribute
    public void onInit(HttpServletRequest request) {
        String uri = request.getRequestURI();
        log.info("请求url--->>>>" + uri);
        Map<String, String[]> parameterMap = request.getParameterMap();
        log.info("请求参数--->>>>"+SlmConstants.gson.toJson(parameterMap));
        Enumeration<String> headerNames = request.getHeaderNames();
        Map<String,String> headers = Maps.newHashMap();
        while (headerNames.hasMoreElements()){
            String headerName = headerNames.nextElement();
            Enumeration<String> headerValues = request.getHeaders(headerName);
            while(headerValues.hasMoreElements()){
                headers.put(headerName, headerValues.nextElement());
            }
        }
        log.info("请求头参数--->>>>"+SlmConstants.gson.toJson(headers));
        String token=request.getHeader(SlmConstants.TOKEN_PREFIX);
        if(!uri.contains("tokens")) {
            if (StringUtils.isEmpty(token)) {
                throw new SysException("token不能为空");
            }
            log.info("获取用户信息");
            NativeWebRequest webRequest = new ServletWebRequest(request);
            Map<String, String> map = (Map<String, String>) webRequest.getAttribute(HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE, RequestAttributes.SCOPE_REQUEST);
            Map<String,String> uriVariables =Maps.newHashMap();
            uriVariables.put("appId",map.get("appId")+"");
            String getUrl =dcClientHost+"/{appId}/users/user-info";
            R<CurrentUser> result = RestTemplateUtils.get(getUrl, new TypeToken<R<CurrentUser>>() {
            }.getType(), uriVariables);
            request.setAttribute(token,SlmConstants.gson.toJson(result.detach()));
        }


    }
    R<Object> call(Action0 action) {
        return NetResponseUtils.call(action);
    }

    <T> R<T> call(Supplier<T> supplier) {
        return NetResponseUtils.call(supplier);
    }

    protected CurrentUser getCurrentUser (HttpServletRequest request){
        String token=request.getHeader(SlmConstants.TOKEN_PREFIX);
        return SlmConstants.gson.fromJson(request.getAttribute(token).toString(),CurrentUser.class);
    }
}
