package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.dao.BusinessConfigMapper;
import com.syc.slm.slmbi.dto.BusinessDTO;
import com.syc.slm.slmbi.entity.BaseEntity;
import com.syc.slm.slmbi.entity.BusinessConfig;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.BusinessConfigService;
import com.syc.slm.slmbi.vo.BusinessConfigVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * slm-bi
 * 系统业务表配置
 * @author : lin
 * @date : 2021-04-20 09:30
 **/
@Service
public class BusinessConfigServiceImpl extends ServiceImpl<BusinessConfigMapper, BusinessConfig> implements BusinessConfigService {
    @Override
    @Transactional(readOnly = true)
    public List<BusinessConfigVo> selectConfigList() {
        LambdaQueryWrapper<BusinessConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        List<BusinessConfig> configs = baseMapper.selectList(where);
        if(CollectionUtils.isNotEmpty(configs)){
            configs.forEach(x->{
                try {
                    if(StringUtils.isNotBlank(x.getField())) {
                        x.setField(URLDecoder.decode(x.getField(), StandardCharsets.UTF_8.name()));
                    }
                }catch (Exception e){
                    e.printStackTrace();
                    throw new SysException("请联系管理员，数据转化出错了");
                }
            });
        }

        return CollectionUtils.isNotEmpty(configs)?SlmConstants.gson.fromJson(SlmConstants.gson.toJson(configs),new TypeToken<List<BusinessConfigVo>>(){}.getType()):
            Lists.newArrayList();
    }

    @Override
    @Transactional(readOnly = true)
    public BusinessConfigVo selectOneConfig(String id) {
        BusinessConfig co = baseMapper.selectById(id);
        try {
            if(StringUtils.isNotBlank(co.getField())) {
                co.setField(URLDecoder.decode(co.getField(), StandardCharsets.UTF_8.name()));
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new SysException("请联系管理员，数据转化出错了");
        }
        return ObjectUtils.isNotEmpty(co)?SlmConstants.gson.fromJson(SlmConstants.gson.toJson(co), BusinessConfigVo.class):null;
    }

    @Override
    @Transactional(readOnly = true)
    public List<Map<String, Object>> getBusinessData(BusinessDTO dto,String uri, CurrentUser user) {
        if(StringUtils.isBlank(dto.getTable())){
            throw new SysException("请传入对应的表名");
        }
        if(StringUtils.isBlank(dto.getStatisticsType())){
            throw new SysException("统计类型不能为空");
        }
        if(StringUtils.isBlank(dto.getX())){
            throw new SysException("x轴维度不能为空");
        }
        if(StringUtils.isBlank(dto.getY())){
            throw new SysException("y轴配置字段不能为空");
        }
        if(CollectionUtils.isEmpty(dto.getWhere())){
            throw new SysException("where 条件不能为空");
        }
        if(dto.getWhere().stream().anyMatch(x -> StringUtils.isBlank(x.getKey()))
           || dto.getWhere().stream().anyMatch(x -> StringUtils.isBlank(x.getValue())) || dto.getWhere().stream().anyMatch(x -> StringUtils.isBlank(x.getQueryType()))) {
            throw new SysException("where 条件不能key value queryType 都不能为空");
        }
        Map<String, String> heads = Maps.newHashMap();
        heads.put("X-Token", user.getToken());
        R<List<Map<String, Object>>> result = RestTemplateUtils.post(uri, heads,dto, new TypeToken<R<List<Map<String, Object>>>>() {}.getType(), null);
        return result.detach();
    }
}
