package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.dto.*;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.VariableService;
import com.syc.slm.slmbi.vo.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * slm-bi
 * 变量
 *
 * @author : lin
 * @date : 2021-03-25 18:36
 **/
@Service
public class VariableServiceImpl implements VariableService {
    @Override
    public IPage<VariableVo> getVariableList(VariableDTO variableDTO,String appId,String token,String uri) {
        IPage<VariableVo> page = new Page<>(variableDTO.getCurrent(),variableDTO.getSize());
        Map<String,String> p = Maps.newHashMap();
        p.put("appId",appId);
        p.put("token",token);
        p.put("from",null);
        p.put("varType",null);
        p.put("type",null);
        p.put("equipmentId",null);
        p.put("customerId",null);
        p.put("name",variableDTO.getVariableName());
        p.put("desc",variableDTO.getDescription());
        p.put("unit",null);
        p.put("keyword",null);
        p.put("isCustomer",null);
        p.put("current",variableDTO.getCurrent()+"");
        p.put("size",variableDTO.getSize()+"");

        R<Map<String, Object>> result = RestTemplateUtils.get(uri, new TypeToken<R<Map<String, Object>>>() {
        }.getType(), p);
        Map<String, Object> detach = result.detach();
        if (CollectionUtils.isNotEmpty(detach)) {
            List<VariableVo> vos = SlmConstants.gsonLowerCaseWithUnderscores.fromJson(SlmConstants.gson.toJson(detach.get("datas")),
                                                                    new TypeToken<List<VariableVo>>() {
                                                                    }.getType());
            page.setTotal((new BigDecimal(detach.get("total") + "")).longValue());
            page.setRecords(vos);
        }
        return page;
    }


    @Override
    public List<VariableSourceVo> getVariableSourceList() {
        List<VariableSourceVo> vos = Lists.newArrayList();
        VariableSourceVo vo = new VariableSourceVo();
        vo.setSourceId("6");
        vo.setSourceName("请选择变量来源");
        VariableSourceVo vo1 = new VariableSourceVo();
        vo1.setSourceId("1");
        vo1.setSourceName("基础采集点");
        VariableSourceVo vo2 = new VariableSourceVo();
        vo2.setSourceId("4");
        vo2.setSourceName("配置变量/dc公式");
        vos.add(vo);
        vos.add(vo1);
        vos.add(vo2);
        return vos;
    }

    @Override
    public IPage<VariableDeviceVo> getVariableDeviceList(String token,String appId,String uri, QueryPageDTO queryPageDTO) {

        IPage<VariableDeviceVo> page = new Page<>(queryPageDTO.getCurrent(), queryPageDTO.getSize());
        Map<String, String> p = Maps.newHashMap();
        p.put("appId", appId);
        p.put("token", token);
        p.put("equipment_info_id", null);
        p.put("customer_id", null);
        p.put("q_name", null);
        p.put("search_code", null);

        p.put("search_model", null);
        p.put("warranty_status", null);
        p.put("page_num", queryPageDTO.getCurrent() + "");
        p.put("page_size", queryPageDTO.getSize() + "");
        R<Map<String, Object>> result = RestTemplateUtils.get(uri, new TypeToken<R<Map<String, Object>>>() {
        }.getType(), p);
        Map<String, Object> detach = result.detach();
        if (CollectionUtils.isNotEmpty(detach)) {
            List<VariableDeviceVo> vos = SlmConstants.gsonLowerCaseWithUnderscores.fromJson(SlmConstants.gson.toJson(detach.get("result")),
                                                                    new TypeToken<List<VariableDeviceVo>>() {
                                                                    }.getType());
            page.setTotal((new BigDecimal(detach.get("total") + "")).longValue());
            page.setRecords(vos);
        }

        return page;
    }

    @Override
    public Map<String, List<Map<String, RealTimeVariableVo>>> getVariableRealTimeDataList(VariableDataDTO variableDTO) {

        return null;
    }

    @Override
    public Map<String, List<Map<String, HistoryVariableVo>>> getVariableHistoryDataList(VariableDataDTO variableDTO) {
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void send(VariableSendDTO sendDTO, CurrentUser currentUser,String uri) {
        if (StringUtils.isBlank(sendDTO.getAccessType())) {
            throw new SysException("请传入访问类型");
        }
        if (StringUtils.isBlank(sendDTO.getEquipmentId())) {
            throw new SysException("请传入设备id");
        }
        if (StringUtils.isBlank(sendDTO.getVariableId())) {
            throw new SysException("请传入变量id");
        }
        if(AccessType.PRIVATE.value.equals(sendDTO.getAccessType())){
            if (StringUtils.isBlank(sendDTO.getUserId())) {
                throw new SysException("访问类型为私有，请传入创建人");
            }
            if(currentUser.getUid().equals(sendDTO.getUserId())){
                throw new SysException("没有访问权限");
            }
        }else if(AccessType.CUSTOM.value.equals(sendDTO.getAccessType())) {
            if(CollectionUtils.isNotEmpty(sendDTO.getPermissions())){
                Map<String, List<CustomPermissionDTO>> maps =
                    sendDTO.getPermissions().stream().collect(Collectors.groupingBy(CustomPermissionDTO::getType));
                List<CustomPermissionDTO> departments = maps.get("department");
                List<CustomPermissionDTO> positions = maps.get("position");
                List<CustomPermissionDTO> users = maps.get("user");
                boolean flag=true;
                if(CollectionUtils.isNotEmpty(users)){
                    if(users.stream().noneMatch(x->currentUser.getUid().equals(x.getBizId()))){
                        flag=false;
                    }
                }
                if(CollectionUtils.isNotEmpty(departments) || CollectionUtils.isNotEmpty(positions)) {
                    R<Map<String, String>> result = setDeptAndRole(currentUser, uri);
                    Map<String, String> detach = result.detach();
                    String officeIds = detach.get("deptIds");
                    String roleIds = detach.get("roleIds");
                    if (flag) {
                        if (CollectionUtils.isNotEmpty(departments)) {
                            if (departments.stream()
                                .map(CustomPermissionDTO::getBizId)
                                .anyMatch(officeIds::contains)) {
                                flag = false;
                            }
                        }
                    }
                    if (flag) {
                        if (CollectionUtils.isNotEmpty(positions)) {
                            if (positions.stream().map(CustomPermissionDTO::getBizId).anyMatch(roleIds::contains)) {
                                flag = false;
                            }
                        }
                    }
                    if(flag){
                        throw new SysException("没有访问权限");
                    }
                }
            }

            //# todo 准备下发接口
        }
    }

    private  R<Map<String,String>> setDeptAndRole(CurrentUser user,String uri){
        Map<String,String> uriVariables =Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        return  RestTemplateUtils.get(uri, new TypeToken<R<Map<String,String>>>() {}.getType(), uriVariables);

    }

}
